package com.gzkx.commonfunction.model;

import lombok.Data;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author adc
 */
@Data
public class ServerMetrics {
    private String serverId;
    private LocalDateTime timestamp;
    // 服务器状态 1 在线 0 下线 Service
    private String status;

    private String errorMessage;

    private String cpuInfo;
    // CPU使用率
    private double cpuUsage;
    // CPU核心数
    private int cpuCores;
    // CPU线程数
    private int cpuThreads;
    // CPU型号
    private String cpuModel;
    // 系统负载
    private double systemLoadAverage;
    // 每个核心的使用率
    private Map<Integer, Double> perCoreUsage = new HashMap<>();

    // 总内存
    private long totalMemory;
    // 已用内存
    private long usedMemory;
    // 内存使用率
    private double memoryUsage;
    
    // 磁盘指标
    private Map<String, DiskMetrics> diskMetrics = new HashMap<>();
    
    // 网络指标
    private Map<String, NetworkMetrics> networkMetrics = new HashMap<>();

    // 总进程数
    private int totalProcesses;
    // 运行中进程数
    private int runningProcesses;
    
    // 系统层面指标
    private String osVersion;
    private String kernelVersion;
    private String timezone;
    private String uptime;
    private String hostname;
    
    // 进程扩展指标
    private Integer zombieProcesses;
    private List<ProcessInfo> topProcesses = new ArrayList<>();
    private Map<String, ServiceStatus> serviceStatus = new HashMap<>();
    
    // 安全相关指标
    private Integer loggedInUsers;
    private Integer failedLoginAttempts;
    private List<String> securityAlerts = new ArrayList<>();
    
    // 容器监控指标
    private List<ContainerInfo> containerInfos = new ArrayList<>();
    
    @Data
    public static class DiskMetrics {
        private String mountPoint;      // 挂载点
        private long totalSpace;        // 总空间
        private long usedSpace;         // 已用空间
        private long diskMargin;         // 剩余空间
        private double usageRate;       // 使用率
        private long readBytes;         // 读取字节数
        private long writeBytes;        // 写入字节数
    }
    
    @Data
    public static class NetworkMetrics {
        private String interfaceName;   // 网卡名称
        private long rxBytes;           // 接收字节数
        private long txBytes;           // 发送字节数
        private long rxPackets;         // 接收数据包数
        private long txPackets;         // 发送数据包数
        private double bandwidth;       // 带宽使用率
    }
    
    @Data
    public static class ProcessInfo {
        private String pid;
        private String user;
        private String command;
        private Double cpuUsage;
        private Double memoryUsage;
        private String startTime;
    }
    
    @Data
    public static class ServiceStatus {
        private String name;
        private String status;
        private String uptime;
        private Boolean isActive;
    }
    
    @Data
    public static class ContainerInfo {
        private String containerId;
        private String name;
        private String image;
        private String status;
        private String created;
        private String cpuUsage;
        private String memoryUsage;
        private String networkUsage;
    }
} 