package com.gzkx.commonfunction.properties;

import lombok.Data;
import org.apache.camel.PropertyInject;

import java.util.List;
import java.util.Map;

/**
 * 监控配置属性类
 * @author adc
 */
@Data
public class MonitorProperties {

    /**
     * 服务器配置列表
     */
    private List<ServerConfig> servers;

    /**
     * 采集配置
     */
    private CollectionConfig collection;

    /**
     * 告警配置
     */
    private AlertConfig alert;

    /**
     * 服务器配置
     */
    @Data
    public static class ServerConfig {

        // 服务器唯一标识
        private String id;
        // 服务器名称
        @PropertyInject("server.name")
        private String name;
        // 服务器类型
        private String serverType;
        // IP地址
        @PropertyInject("server.host")
        private String host;
        // SSH端口
        @PropertyInject("server.port")
        private int port;
        // SSH用户名
        @PropertyInject("server.username")
        private String userName;
        // SSH密码
        @PropertyInject("server.password")
        private String password;
        // Docker守护进程地址
        @PropertyInject("server.password")
        private String dockerHost;
        // 服务器标签
        private Map<String, String> tags;
    }

    /**
     * 采集配置
     */
    @Data
    public static class CollectionConfig {
        @PropertyInject("server.interval.collectTime")
        private int interval = 60;  // 采集间隔（秒）
        @PropertyInject("server.interval.timeout")
        private int timeout = 30;   // 采集超时时间（秒）
    }

    /**
     * 告警配置
     */
    @Data
    public static class AlertConfig {
        @PropertyInject("server.cpuThreshold")
        private double cpuThreshold = 80.0;    // CPU告警阈值（%）
        @PropertyInject("server.memoryThreshold")
        private double memoryThreshold = 80.0;  // 内存告警阈值（%）
        @PropertyInject("server.diskThreshold")
        private double diskThreshold = 80.0;    // 磁盘告警阈值（%）
        @PropertyInject("server.gj.interval")
        private int interval = 300;             // 告警间隔（秒）
    }
} 