package com.gzkx.commonfunction.utils;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.security.Keys;

import javax.crypto.SecretKey;
import java.util.*;

public class JWTUtil {

    public static SecretKey generateSecretKey() {
        return Jwts.SIG.HS256.key().build();
    }

    public static String generateToken(String username, List<String> roles, SecretKey key) {
        return Jwts.builder()
                .subject(username)
                .id(UUID.randomUUID().toString())
                .claim("roles", roles)
                .issuedAt(new Date())
                .expiration(new Date(System.currentTimeMillis() + 3600000)) // 1小时过期
                .signWith(key)
                .compact();
    }

    public static Claims validateToken(String token, SecretKey key) {
        // SecretKey key = Keys.hmacShaKeyFor(secret.getBytes());
        return Jwts.parser()
                .verifyWith(key)
                .build()
                .parseSignedClaims(token)
                .getPayload();
    }

    public static void main(String[] args) {
//        SecretKey secretKey = generateSecretKey();
        SecretKey secretKey = Keys.hmacShaKeyFor("4a5e8d7c3b2f1a9e6d8c7b5a4e3f2d1c0b9a8d7e6f5a4b3c2d1e0f9a8b7c6d5e4f3a2b1".getBytes());
        String token = generateToken("ConnorFu", List.of("ROLE_ADMIN", "ROLE_USER"), secretKey);
        System.out.println("生成Token: " + token);
        List<?> roles = new ArrayList<>();
        try {
            Claims claims = validateToken("eyJhbGciOiJIUzUxMiJ9.eyJzdWIiOiIxODkxNzcyNDY1NDI3MzUzNjAyIiwidXNlcklkIjoiMTg5MTc3MjQ2NTQyNzM1MzYwMiIsInJvbGVzIjpbInJvbGVfMDAxIiwicm9sZV8wMDIiXSwiaWF0IjoxNzQyODA4NzAxLCJleHAiOjE3NDI4MTIzMDF9.EOTcz-q0s8qI2YBKK2G0LW5y9pwletDVkXVgQ_vTNSCvH7o1YpbippOfUufNCWVyc0b3WRRTaopHFDFglxmqlQ", secretKey);
            roles = claims.get("roles", List.class);
        } catch (ExpiredJwtException e) {
            System.out.println("Token已过期: " + e.getMessage());
        }
        if (!roles.contains("role_001")) {
            throw new RuntimeException("用户无权限");
        }
        System.out.println("解析Token: " + roles);

    }

}
