package com.gzkx.commonfunction.utils.sso;

import com.fasterxml.jackson.databind.ObjectMapper;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;

import java.security.PrivateKey;
import java.security.PublicKey;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * JWT 工具类
 * @author adc
 * @since 2025-03-28
 * @version 1.0
 */
public class JwtUtils {

    // AES 密钥（16字节）
    private static final String AES_SECRET = "pk187fm710asoij6";

    /**
     *
     * @param payload 加密的数据载荷
     * @param privateKey 私钥加签key
     * @param expireMinutes 分钟
     * @return 返回加密token
     * @throws Exception 异常....
     */
    public static String generateEncryptedToken(Map<String, Object> payload, PrivateKey privateKey, int expireMinutes) throws Exception {
        // 把 payload 转成 JSON 字符串
        String json = new ObjectMapper().writeValueAsString(payload);

        // 用 AES 加密
        String encryptedPayload = AesUtils.encrypt(json, AES_SECRET);

        // 构建 JWT
        return Jwts.builder()
                .id(UUID.randomUUID().toString())
                // 把密文放进字段 data
                .claim("data", encryptedPayload)
                .expiration(Date.from(LocalDateTime.now().plusMinutes(expireMinutes).atZone(ZoneId.systemDefault()).toInstant()))
                .signWith(privateKey, Jwts.SIG.RS256)
                .compact();
    }


    // 新版解析token的方法
    public   static Map<String, Object> parseToken(String token,String minioHost,String accessKey,String secretKey,String bucketName,String objectName) throws Exception {
        PublicKey publicKey;
        try {
            publicKey  = KeyUtils.getPublicKeyFromMinio(
                    minioHost, accessKey, secretKey, bucketName, objectName
            );
        } catch (Exception e) {
            throw new RuntimeException("获取公钥失败",e);
        }

        Claims claims = Jwts.parser()
                .verifyWith(publicKey)
                .build()
                .parseSignedClaims(token)
                .getPayload();

        // 获取加密后的数据
        String encryptedData = claims.get("data", String.class);

        // 解密 AES
        String json = AesUtils.decrypt(encryptedData, AES_SECRET);

        // 转换成 Map
        return new ObjectMapper().readValue(json, Map.class);
    }


    /**
     *
     * @param token token
     * @param publicKey 公钥解密key
     * @return 返回解密后的数据
     * @throws Exception 异常。。。
     */
    public static Map<String, Object> parseEncryptedToken(String token, PublicKey publicKey) throws Exception {
        Claims claims = Jwts.parser()
                .verifyWith(publicKey)
                .build()
                .parseSignedClaims(token)
                .getPayload();

        // 获取加密后的数据
        String encryptedData = claims.get("data", String.class);

        // 解密 AES
        String json = AesUtils.decrypt(encryptedData, AES_SECRET);

        // 转换成 Map
        return new ObjectMapper().readValue(json, Map.class);
    }

    /**
     * 获取用户账号
     */
    public static String getAccountNumber(String token,String minioHost,String accessKey,String secretKey,String bucketName,String objectName) throws Exception {
        PublicKey publicKey;
        try {
            publicKey  = KeyUtils.getPublicKeyFromMinio(
                    minioHost, accessKey, secretKey, bucketName, objectName
            );
        } catch (Exception e) {
            throw new RuntimeException("获取公钥失败",e);
        }
        Map<String, Object> payload = parseEncryptedToken(token, publicKey);
        return (String) payload.get("accountNumber");
    }

    /**
     * 获取用户ID
     */
    public static String getUserId(String token,String minioHost,String accessKey,String secretKey,String bucketName,String objectName) throws Exception {
        PublicKey publicKey;
        try {
            publicKey  = KeyUtils.getPublicKeyFromMinio(
                    minioHost, accessKey, secretKey, bucketName, objectName
            );
        } catch (Exception e) {
            throw new RuntimeException("获取公钥失败",e);
        }
        Map<String, Object> payload = parseEncryptedToken(token, publicKey);
        return (String) payload.get("userId");
    }


    /**
     * 获取自定义字段
     */
    public static String getCustomField(String token, String fieldName,String minioHost,String accessKey,String secretKey,String bucketName,String objectName) throws Exception {
        PublicKey publicKey;
        try {
            publicKey  = KeyUtils.getPublicKeyFromMinio(
                    minioHost, accessKey, secretKey, bucketName, objectName
            );
        } catch (Exception e) {
            throw new RuntimeException("获取公钥失败",e);
        }
        Map<String, Object> payload = parseEncryptedToken(token, publicKey);
        Object value = payload.get(fieldName);
        return value != null ? value.toString() : null;
    }


    /**
     * 获取 JWT 的过期时间
     */
    public static Date getExpiration(String token,String minioHost,String accessKey,String secretKey,String bucketName,String objectName) {
        PublicKey publicKey;
        try {
            publicKey  = KeyUtils.getPublicKeyFromMinio(
                    minioHost, accessKey, secretKey, bucketName, objectName
            );
        } catch (Exception e) {
            throw new RuntimeException("获取公钥失败",e);
        }
        Claims claims = Jwts.parser()
                .verifyWith(publicKey)
                .build()
                .parseSignedClaims(token)
                .getPayload();

        return claims.getExpiration();
    }


    public static void main(String[] args) throws Exception {
        String minioEndpoint = "http://192.168.1.101:19000";
        String accessKey = "admin";
        String secretKey = "admin123456";
        String bucketName = "keypair";
        String objectName = "private_key.der";
        PrivateKey privateKey;
        // 从 MinIO 获取公钥
        try {
            privateKey = KeyUtils.getPrivateKeyFromMinio(
                    minioEndpoint, accessKey, secretKey, bucketName, objectName
            );
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        Map<String,Object> map = new HashMap<>();
        map.put("accountNumber", "user");
        map.put("userId", "user_001");
        String token = generateEncryptedToken(map, privateKey, 60*24);
        // 加密token
        System.out.println(token);
        // 解密token
        Map<String, Object> payload = null;
        try {
            payload = parseToken(token, minioEndpoint, accessKey, secretKey, bucketName, "public_key.der");
        } catch (Exception e) {
            throw new RuntimeException("非法token");
        }
        String accountNumber = payload.get("accountNumber").toString();
        String userId = payload.get("userId").toString();
        System.out.println("用户账户：" + accountNumber + " 用户ID：" + userId);
    }



}
